@@ -34,24 +34,32 @@
 // Remarks: 
 //
 // History: 03/22/97 - MAR - created.
+//          01/26/04 - STF - Modified to pass information to
+//		       Automatic Schematics Generation library
+//		       linked to xcircuit application. Addition of
+//		       Add2TermModule(), Add3TermModule(), etc.
 //
 // Copyright (c) 1997 Michael A. Riepe
+// Copyright (c) 2004 Stephen T. Frezza
 //
 // RCS Version:
 //     $Id: device.cc,v 1.1.1.1 2002/05/01 14:23:27 mguthaus Exp $
 //-----------------------------------------------------------------------------
 
 #include <stdio.h>
+#include <stdarg.h>
 #include <string.h>
 #include <map>
 #include <vector>
 #include <list>
 using namespace std;
-
 #include "defs.hh"
 #include "utils.hh"
 #include "device.hh"
 
+extern "C" void Add2TermModule(char *, char *, char *, char *);
+extern "C" void Add3TermModule(char *, char *, char *, char *, char *);
+extern "C" void AddNTermModule(char *, char *, int, ...);
 
 // ****************************************************************************
 // class: device
@@ -249,7 +257,10 @@ void device::copy_parameters(map<char*, char*, ltstr> Sp) {
 //-----------------------------------------------------------------------------
 
 port *device::add_port(char *port_name) {
-    DBG0(cout<<"adding port "<<port_name<<" to device "<<name()<<endl;);
+    DBG0(cout<<"adding port "<<port_name<<" to deFinished parsing input. Elapsed time: 0
+        Finished flattening netlist. Elapsed time: 0
+Placing internal modules...Segmentation fault
+vice "<<name()<<endl;);
 
     char *new_name = strdup(port_name);
     port *new_port = new port(new_name);
@@ -446,6 +457,7 @@ void jfet::print(ostream& os) {
 	os << endl;
 }
 void mosfet::print(ostream& os) {
+/*
 	os << name() << " ";
 	os << *_drain << " " << *_gate << " ";
 	os << *_source << " ";
@@ -453,48 +465,159 @@ void mosfet::print(ostream& os) {
 	os << _model << " ";
 	dump_params(os);
 	os << endl;
+*/
+	char* pName = name();
+
+	net* pNet1Ref = _drain->net_ref();
+	char* pNode1 = pNet1Ref->name();
+
+	net* pNet2Ref = _gate->net_ref();
+	char* pNode2 = pNet2Ref->name();
+        
+	net* pNet3Ref = _source->net_ref();
+	char* pNode3 = pNet3Ref->name();
+
+	net* pNet4Ref = _bulk->net_ref();
+	char* pNode4 = pNet4Ref->name();
+	// This is not quite right.  Sometimes you'll want a four-terminal module here -STF
+	Add3TermModule( pName, "MSFET" , pNode1, pNode2, pNode3);
+
+//	if (_bulk && _bulk->net_ref()) {
+//	    AddNTermModule( pName, "MSFET" , 4, "D", pNode1, "G", pNode2, "S", pNode3,
+//			"B", pNode4);
+//	else
+//	    AddNTermModule( pName, "MSFET" , 3, "D", pNode1, "G", pNode2, "S", pNode3); */
+
 }
 void diode::print(ostream& os) {
+
+/*
 	os << name() << " ";
 	os << *_nplus << " " << *_nminus << " ";
 	dump_params(os);
 	os << endl;
+*/
+	char* pName = name();
+	Add2TermModule( pName, "RESTR" , (char*)(_nplus), (char*)(_nminus));
+
+//	AddNTermModule( pName, "RESTR" , 2, "1", (char*)(_nplus), "2",
+//		(char*)(_nminus));
 }
 void resistor::print(ostream& os) {
+
+/*
 	os << name() << " ";
 	os << *_node1 << " " << *_node2 << " ";
 	dump_params(os);
 	os << endl;
+*/
+	char* pName = name();
+
+	net* pNet1Ref = _node1->net_ref();
+	char* pNode1 = pNet1Ref->name();
+
+	net* pNet2Ref = _node2->net_ref();
+	char* pNode2 = pNet2Ref->name();
+        
+
+	Add2TermModule( pName, "RESTR" , pNode1, pNode2);
+//	AddNTermModule( pName, "RESTR" , 2, "1", pNode1, "2", pNode2);
 }
 void capacitor::print(ostream& os) {
+
+/*
 	os << name() << " ";
 	os << *_node1 << " " << *_node2 << " ";
 	dump_params(os);
 	os << endl;
+*/
+	char* pName = name();
+
+	net* pNet1Ref = _node1->net_ref();
+	char* pNode1 = pNet1Ref->name();
+
+	net* pNet2Ref = _node2->net_ref();
+	char* pNode2 = pNet2Ref->name();
+
+	Add2TermModule( pName, "CAPC" , pNode1, pNode2 );
+//	AddNTermModule( pName, "CAPC" , 2, "1", pNode1, "2", pNode2 );
 }
 void coupling::print(ostream& os) {
+
+/*
 	os << name() << " ";
 	os << *_inductor1 << " " << *_inductor2 << " ";
 	dump_params(os);
 	os << endl;
+*/
+	char* pName = name();
+	Add2TermModule( pName, "RESTR" , (char*)(_inductor1), (char*)(_inductor2));
+
+//	AddNTermModule( pName, "RESTR" , 2, "1", (char*)(_inductor1),
+//		"2", (char*)(_inductor2));
+
 }
 void inductor::print(ostream& os) {
+/*
 	os << name() << " ";
 	os << *_node1 << " " << *_node2 << " ";
 	dump_params(os);
 	os << endl;
+*/
+	char* pName = name();
+	
+	net* pNet1Ref = _node1->net_ref();
+	char* pNode1 = pNet1Ref->name();
+
+	net* pNet2Ref = _node2->net_ref();
+	char* pNode2 = pNet2Ref->name();
+
+
+	Add2TermModule( pName, "INDR" , pNode1, pNode2 );
+
+//	AddNTermModule( pName, "INDR" , 2, "1", pNode1, "2", pNode2 );
+
 }
 void vsource::print(ostream& os) {
+/*
 	os << name() << " ";
 	os << *_plus_node << " " << *_minus_node << " ";
 	dump_params(os);
 	os << endl;
+*/
+	char* pName = name();
+
+	net* pNet1Ref = _plus_node->net_ref();
+	char* pNode1 = pNet1Ref->name();
+
+	net* pNet2Ref = _minus_node->net_ref();
+	char* pNode2 = pNet2Ref->name();
+
+
+	Add2TermModule( pName, "VAMP" , pNode1, pNode2 );
+
+//	AddNTermModule( pName, "VAMP" , 2, "P", pNode1, "M", pNode2 );
 }
 void isource::print(ostream& os) {
+/*
 	os << name() << " ";
 	os << *_plus_node << " " << *_minus_node << " ";
 	dump_params(os);
 	os << endl;
+*/
+
+	char* pName = name();
+	
+	net* pNet1Ref = _plus_node->net_ref();
+	char* pNode1 = pNet1Ref->name();
+
+	net* pNet2Ref = _minus_node->net_ref();
+	char* pNode2 = pNet2Ref->name();
+
+
+	Add2TermModule( pName, "VAMP" , pNode1, pNode2 );
+
+//	AddNTermModule( pName, "VAMP" , 2, "P", pNode1, "M", pNode2 );
 }
 
 
