@@ -52,7 +52,7 @@ extern char _STR[150];
 /*----------------------------------------------------------------------*/
 
 u_char param_select[] = {
-   POLYGON | SPLINE | LABEL | OBJECT | ARC,	/* P_POSITION */
+   SEL_ANY,					/* P_NUMERIC */
    LABEL,					/* P_SUBSTRING */
    POLYGON | SPLINE | LABEL | OBJECT | ARC,	/* P_POSITION_X */
    POLYGON | SPLINE | LABEL | OBJECT | ARC,	/* P_POSITION_Y */
@@ -65,7 +65,8 @@ u_char param_select[] = {
    LABEL | OBJECT,				/* P_ROTATION */
    LABEL | OBJECT,				/* P_SCALE */
    POLYGON | SPLINE | ARC | PATH,		/* P_LINEWIDTH */
-   SEL_ANY 					/* P_COLOR (tbd) */
+   SEL_ANY, 					/* P_COLOR (tbd) */
+   POLYGON | SPLINE | LABEL | OBJECT | ARC 	/* P_POSITION */
 };
 
 #ifdef TCL_WRAPPER
@@ -74,7 +75,7 @@ xcWidget *param_buttons[] = {
 };
 #else
 Widget *param_buttons[] = {
-   &ParametersPositionButton,		/* P_POSITION */
+   &ParametersNumericButton,		/* P_NUMERIC */
    &ParametersSubstringButton,		/* P_SUBSTRING */
    &ParametersPositionButton,		/* P_POSITION_X */
    &ParametersPositionButton,		/* P_POSITION_Y */
@@ -87,7 +88,8 @@ Widget *param_buttons[] = {
    &ParametersRotationButton,		/* P_ROTATION */
    &ParametersScaleButton,		/* P_SCALE */
    &ParametersLinewidthButton,		/* P_LINEWIDTH */
-/* &ParametersColorButton (does not exist yet) P_COLOR */
+   &ParametersColorButton,		/* P_COLOR */
+   &ParametersPositionButton,		/* P_POSITION */
 };
 #endif
 
@@ -145,6 +147,22 @@ eparamptr make_new_eparam(char *key)
    return newepp;
 }
 
+/*----------------------------------------------------------------------*/
+/* Determine if a parameter is indirectly referenced.  If so, return	*/
+/* the reference key.  If not, return NULL.				*/
+/*----------------------------------------------------------------------*/
+
+char *find_indirect_param(objinstptr thisinst, char *key)
+{
+   eparamptr epp;
+
+   for (epp = thisinst->passed; epp != NULL; epp = epp->next) {
+      if (!strcmp(epp->key, key))
+	 return epp->pdata.refkey;
+   }
+   return NULL;
+}
+
 /*----------------------------------------------------------------------*/
 /* Find the parameter in the indicated object by key			*/
 /*----------------------------------------------------------------------*/
@@ -324,14 +342,14 @@ void setparammarks(genericptr thiselem)
 
    /* Reset all variables to false */
    for (i = 0; i < 14; i++)
-      Tcl_SetVar(xcinterp, (char *)paramvars[i], "false", TCL_NAMESPACE_ONLY);
+      Tcl_SetVar2(xcinterp, "XCOps", (char *)paramvars[i], "false", TCL_NAMESPACE_ONLY);
 
    /* For each parameter declared, set the corresponding Tcl variable */
    if (thiselem != NULL) {
       for (epp = thiselem->passed; epp != NULL; epp = epp->next) {
 	 ops = match_param(topobject, epp->key);
 	 if (ops == NULL) continue;	/* error condition */
-         Tcl_SetVar(xcinterp, (char *)paramvars[ops->which], "true",
+         Tcl_SetVar2(xcinterp, "XCOps", (char *)paramvars[ops->which], "true",
 		TCL_NAMESPACE_ONLY);
       }
    }
@@ -400,6 +418,7 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
    XPoint *setpt;
    Boolean needrecalc;	/* for arcs and splines */
    int retval = -1;
+   char *promoted;
 
    for (eptr = thisobj->plist; eptr < thisobj->plist + thisobj->parts; eptr++) {
 
@@ -413,8 +432,35 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 	 ops = match_param(thisobj, epp->key);
 
 	 if (ips != NULL) {
-	    ival = ips->parameter.ivalue;
-	    fval = ips->parameter.fvalue;
+
+	    /* Get integer and float values.  Promote types if necessary */
+
+	    switch(ips->type) {
+	       case XC_INT:
+		  ival = ips->parameter.ivalue;
+		  fval = (float)(ival);
+		  break;
+	       case XC_FLOAT:
+		  fval = ips->parameter.fvalue;
+		  ival = (int)(fval + 0.5);
+		  break;
+	       case XC_STRING:
+		  promoted = textprint(ips->parameter.string, pinst);
+		  if (sscanf(promoted, "%g", &fval) == 1)
+		     ival = (int)(fval + 0.5);
+		  else {
+		     free(promoted);
+		     continue;
+		  }
+		  free(promoted);
+		  break;
+	       case XC_EXPR:
+		  if ((promoted = evaluate_expr(ips)) == NULL) continue;
+		  if (sscanf(promoted, "%g", &fval) == 1)
+		     ival = (int)(fval + 0.5);
+		  free(promoted);
+		  break;
+	    }
 	 }
 	 else if (ops == NULL)
 	    continue;
@@ -426,19 +472,15 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 	       oparamptr refop, newop;
 
 	       thisinst = TOOBJINST(eptr);
-	       /* Sanity checks: refkey must exist in object */
+
+	       /* Sanity check: refkey must exist in object */
 	       refop = match_param(thisinst->thisobject, key);
 	       if (refop == NULL) {
 		  Fprintf("Error:  Reference key %s does not exist in object %s\n",
 				key, thisinst->thisobject->name);
 		  continue;
 	       }
-	       /* "type" and "which" records must match */
-	       else if ((ops->which != refop->which) || (ops->type != refop->type)) {
-		  Fprintf("Error:  Reference parameter %s is different type "
-				"than source parameter %s\n", key, refop->key);
-		  continue;
-	       }
+
 	       /* If an instance value already exists, remove it */
 	       newop = match_instance_param(thisinst, refop->key);
 	       if (newop != NULL)
@@ -684,7 +726,7 @@ void pwriteback(objinstptr thisinst)
       float fval;
    } wtemp;
 
-   pinst = (thisinst == areastruct.topinstance) ? areastruct.topinstance : thisinst;
+   pinst = thisinst;
    thisobj = (pinst == NULL) ? topobject : pinst->thisobject;
 
    /* Make sure that all instance values exist */
@@ -697,7 +739,10 @@ void pwriteback(objinstptr thisinst)
    /* value of any one parameter on a single call to pwriteback().		*/
 
    for (ops = thisobj->params; ops != NULL; ops = ops->next) {
-      if (ops->which == P_SUBSTRING) continue;	/* handle numerical params only */
+      /* handle pre-assigned numeric parameters only */
+      if ((ops->which == P_SUBSTRING) || (ops->which == P_EXPRESSION) ||
+		(ops->which == P_NUMERIC))
+	 continue;
       found = 0;
       changed = FALSE;
       ips = (pinst != NULL) ? match_instance_param(pinst, ops->key) : NULL;
@@ -888,6 +933,12 @@ void replaceparams(objinstptr thisinst)
 	       free_instance_param(thisinst, ips);
 	    }
 	    break;
+	 case XC_EXPR:
+	    if (strcmp(ops->parameter.expr, ips->parameter.expr)) {
+	       free(ops->parameter.expr);
+	       ops->parameter.expr = ips->parameter.expr;
+	       free_instance_param(thisinst, ips);
+	    }
 	 case XC_INT: case XC_FLOAT:
 	    if (ops->parameter.ivalue != ips->parameter.ivalue) {
 	       ops->parameter.ivalue = ips->parameter.ivalue;
@@ -926,19 +977,32 @@ void resolveparams(objinstptr thisinst)
 	    break;
 
       if ((spec == NULL) || (spec->virtual == FALSE)) {
-         /* Fprintf(stdout, "Came from library or top page:  "
-			"changing default value\n"); */
+         /* Fprintf(stdout, "Came from library:  changing default value\n"); */
          replaceparams(thisinst);
          return;
       }
    }
 
+   /* Parameters which are changed on a top-level page must also change	*/
+   /* the default value; otherwise, the instance value shadows the page	*/
+   /* object's value but the page object's value is the one written to	*/
+   /* the output file.							*/
+
+   else if (is_page(thisinst->thisobject) >= 0) {
+      replaceparams(thisinst);
+      return;
+   }
+
    thisobj = thisinst->thisobject;
 
    for (ops = thisobj->params; ops != NULL; ops = ops->next) {
       ips = match_instance_param(thisinst, ops->key);
       if (ips == NULL) continue;  /* this parameter is already default */
 
+      /* If type or which fields do not match, then we don't need to look */
+      /* any further;  object and instance have different parameters.	  */
+      if ((ips->type != ops->type) || (ips->which != ops->which)) continue;
+
       switch(ops->type) {
 	 case XC_STRING:
 	    if (!stringcomp(ops->parameter.string, ips->parameter.string)) {
@@ -946,6 +1010,12 @@ void resolveparams(objinstptr thisinst)
 	       free_instance_param(thisinst, ips);
 	    }
 	    break;
+	 case XC_EXPR:
+	    if (!strcmp(ops->parameter.expr, ips->parameter.expr)) {
+	       free(ips->parameter.expr);
+	       free_instance_param(thisinst, ips);
+	    }
+	    break;
 	 case XC_INT: case XC_FLOAT:
 	    if (ops->parameter.ivalue == ips->parameter.ivalue) {
 	       free_instance_param(thisinst, ips);
@@ -962,6 +1032,37 @@ void resolveparams(objinstptr thisinst)
    }
 }
 
+/*--------------------------------------------------------------*/
+/* Return a copy of the single eparameter "cepp"		*/
+/*--------------------------------------------------------------*/
+
+eparamptr copyeparam(eparamptr cepp, genericptr thiselem)
+{
+   eparamptr newepp;
+
+   newepp = make_new_eparam(cepp->key);
+   if ((thiselem->type == OBJECT) && (cepp->pdata.refkey != NULL))
+      newepp->pdata.refkey = strdup(cepp->pdata.refkey);
+   else
+      newepp->pdata.pointno = cepp->pdata.pointno;
+   return newepp;
+}
+
+/*------------------------------------------------------*/
+/* Copy all element parameters from source to dest	*/
+/*------------------------------------------------------*/
+
+void copyalleparams(genericptr destinst, genericptr sourceinst)
+{
+   eparamptr cepp, newepp;
+
+   for (cepp = sourceinst->passed; cepp != NULL; cepp = cepp->next) {
+      newepp = copyeparam(cepp, sourceinst);
+      newepp->next = destinst->passed;
+      destinst->passed = newepp;
+   }
+}
+
 /*--------------------------------------------------------------*/
 /* Return a copy of the single parameter "cops"			*/
 /*--------------------------------------------------------------*/
@@ -971,13 +1072,15 @@ oparamptr copyparameter(oparamptr cops)
    oparamptr newops;
 
    newops = make_new_parameter(cops->key);
-   newops->next = NULL;
    newops->type = cops->type;
    newops->which = cops->which;
    switch(cops->type) {
       case XC_STRING:
 	 newops->parameter.string = stringcopy(cops->parameter.string);
 	 break;
+      case XC_EXPR:
+	 newops->parameter.expr = strdup(cops->parameter.expr);
+	 break;
       case XC_INT: case XC_FLOAT:
 	 newops->parameter.ivalue = cops->parameter.ivalue;
 	 break;
@@ -1013,6 +1116,78 @@ void copyparams(objinstptr destinst, objinstptr sourceinst)
    }
 }
 
+/*--------------------------------------------------------------*/
+/* Make an unreferenced parameter expression in the object	*/
+/* refobject.							*/
+/* The expression may have either a numeric or string result.	*/
+/* the proper "which" value is passed as an argument.		*/
+/*--------------------------------------------------------------*/
+
+void makeexprparam(objectptr refobject, char *key, char *value, int which)
+{
+   oparamptr newops;
+
+   /* Ensure that no two parameters have the same name! */
+
+   if (check_param(refobject, key)) {
+      Wprintf("There is already a parameter with that key!");
+   }
+
+   newops = make_new_parameter(key);
+   newops->next = refobject->params;
+   refobject->params = newops;
+   newops->type = XC_EXPR;		/* expression requiring evaluation */
+   newops->which = which;
+   newops->parameter.expr = strdup(value);
+   incr_changes(refobject);
+}
+
+/*------------------------------------------------------------------*/
+/* Make an unreferenced numerical parameter in the object refobject */
+/*------------------------------------------------------------------*/
+
+void makefloatparam(objectptr refobject, char *key, float value)
+{
+   oparamptr newops;
+
+   /* Ensure that no two parameters have the same name! */
+
+   if (check_param(refobject, key)) {
+      Wprintf("There is already a parameter with that key!");
+   }
+
+   newops = make_new_parameter(key);
+   newops->next = refobject->params;
+   refobject->params = newops;
+   newops->type = XC_FLOAT;		/* general-purpose numeric */
+   newops->which = P_NUMERIC;
+   newops->parameter.fvalue = value;
+   incr_changes(refobject);
+}
+
+/*----------------------------------------------------------------*/
+/* Make an unreferenced string parameter in the object refobject. */
+/*----------------------------------------------------------------*/
+
+void makestringparam(objectptr refobject, char *key, stringpart *strptr)
+{
+   oparamptr newops;
+
+   /* Ensure that no two parameters have the same name! */
+
+   if (check_param(refobject, key)) {
+      Wprintf("There is already a parameter with that key!");
+   }
+
+   newops = make_new_parameter(key);
+   newops->next = refobject->params;
+   refobject->params = newops;
+   newops->type = XC_STRING;
+   newops->which = P_SUBSTRING;
+   newops->parameter.string = strptr;
+   incr_changes(refobject);
+}
+
 /*--------------------------------------------------------------*/
 /* Make a numerical (integer or float) parameter.		*/
 /* If "key" is non-NULL, then the parameter key will be set	*/
@@ -1047,7 +1222,7 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key)
       return;
    }
 
-   /* Cannot form a parameter on a top-level page */
+   /* Cannot form this type of parameter on a top-level page */
    if (is_page(topobject) != -1) {
       Wprintf("Cannot form a parameter in a top-level page!");
       return;
@@ -1326,6 +1501,26 @@ void insertparam()
       return;
    }
 
+#ifdef TCL_WRAPPER
+
+   /* The Tcl version uses a GUI selection mechanism. */
+
+   result = Tcl_Eval(xcinterp, "xcircuit::promptselectparam");
+   if (result != TCL_OK) {
+      Tcl_SetResult(xcinterp, "Error in executing promptselectparam", NULL);
+      return;
+   }
+   selparm = Tcl_GetString(Tcl_GetObjResult(xcinterp));
+
+   ops = match_param(topobject, selparm);
+   if (ops == NULL)
+   {
+       Tcl_SetResult(xcinterp, "No such parameter.", NULL);
+       selparm = NULL;
+   }
+
+#else
+
    /* First pass is to check how many string parameters there are:	*/
    /* If only one, then automatically use it.  Otherwise, prompt	*/
    /* for which parameter to use.					*/
@@ -1358,37 +1553,11 @@ void insertparam()
       Wprintf(sstart);
       free(sstart);
 
-#ifdef TCL_WRAPPER
-
-      /* The Tcl version uses a GUI selection mechanism. */
-
-      result = Tcl_Eval(xcinterp, "xcircuit::promptselectparam");
-      if (result != TCL_OK) {
-	 Tcl_SetResult(xcinterp, "Error in executing promptselectparam", NULL);
-         return;
-      }
-      selparm = Tcl_GetString(Tcl_GetObjResult(xcinterp));
-
-      ops = match_param(topobject, selparm);
-      if (ops == NULL)
-      {
-	 Tcl_SetResult(xcinterp, "No such parameter.", NULL);
-	 selparm = NULL;
-      }
-      else if (ops->type != XC_STRING)
-      {
-	 Tcl_SetResult(xcinterp, "The parameter is not a substring.", NULL);
-	 selparm = NULL;
-      }
-      /* Relate index in list (string parameters only) to index	*/
-      /* over all parameters.					*/
-
-#else
       /* Need a new non-Tcl version method . . . */
       ops = NULL;	/* placeholder */
+   }
 
 #endif
-   }
 
    if (ops != NULL)
       labeltext(PARAM_START, selparm);
@@ -1479,6 +1648,8 @@ void destroyinst(objinstptr tinst, objectptr refobj, char *key)
       if (ops != NULL) {
 	 if (ops->type == XC_STRING)
 	    freelabel(ops->parameter.string);
+	 else if (ops->type == XC_EXPR)
+	    free(ops->parameter.expr);
 	 free_instance_param(tinst, ops);
       }
    }
@@ -1533,9 +1704,10 @@ void free_object_param(objectptr thisobj, oparamptr thisparam)
             searchinst(*(xobjs.userlibs[j].library + k), thisobj, key);
       }
    }
-   for (k = 0; k < xobjs.delbuffer.number; k++) {
-      searchinst(*(xobjs.delbuffer.library + k), thisobj, key);
-   }
+
+   /* Ensure that this parameter is not referred to in the undo records */
+   /* We could be kinder and gentler to the undo record here. . . */
+   flush_undo_stack();
 
    /* Also check through all instances on the library page */
    if (l >= 0)
@@ -1814,6 +1986,8 @@ void removeinstparams(objinstptr thisinst)
       free(ops->key);
       if (ops->type == XC_STRING)
 	 freelabel(ops->parameter.string);
+      else if (ops->type == XC_EXPR)
+	 free(ops->parameter.expr);
       free(ops);
    }
 }
@@ -1832,6 +2006,8 @@ void removeparams(objectptr thisobj)
       free(ops->key);
       if (ops->type == XC_STRING)
 	 freelabel(ops->parameter.string);
+      else if (ops->type == XC_EXPR)
+	 free(ops->parameter.expr);
       free(ops);
    }
 }
