@@ -185,6 +185,10 @@ extern int lookdirectory(char *);
 
 /* from files.c: */
 
+#ifdef ASG
+extern void importspice();
+#endif
+
 extern char *ridnewline(char *);
 extern void free_single(genericptr *);
 extern void reset(objectptr, short);
@@ -621,15 +625,13 @@ extern short readcommand(short, FILE *);
 extern void ghostinit();
 extern void send_client(Atom);
 extern void ask_for_next();
-extern void wait_for_page();
-extern void wait_for_done();
 extern void start_gs();
 extern void parse_bg(FILE *, FILE *);
 extern void bg_get_bbox();
 extern void backgroundbbox(int);
 extern void readbackground(FILE *);
 extern void savebackground(FILE *, char *);
-extern void call_loadbg(char *);
+extern void register_bg(char *);
 extern void loadbackground();
 extern void send_to_gs(char *);
 extern int renderbackground();
@@ -637,6 +639,10 @@ extern int copybackground();
 extern int exit_gs();
 extern int reset_gs();
 
+#ifndef TCL_WRAPPER
+extern Boolean render_client();
+#endif
+
 /* from schema.c: */
 
 extern objectptr NameToObject(char *, objinstptr *, Boolean);
