@@ -52,7 +52,6 @@ extern char	  _STR[150];
 extern char	  _STR2[250];
 
 /*-------------------------------------------------------------------------*/
-#ifdef SCHEMA
 
 extern xcWidget wsymb, wschema;
 
@@ -60,23 +59,24 @@ extern xcWidget wsymb, wschema;
 /* Menu calls (procedure wrappers)			  */
 /*--------------------------------------------------------*/
 
-void callgennet(xcWidget w, pointertype mode, caddr_t calldata)
+void callwritenet(xcWidget w, pointertype mode, caddr_t calldata)
 {
    switch(mode) {
      case 0:
-        gennet("spice", "spc");
+        writenet(topobject, "spice", "spc");
 	break;
      case 1:
-        gennet("sim", "sim");
+        writenet(topobject, "sim", "sim");
 	break;
      case 2:
-        gennet("pcb", "pcbnet");
+        writenet(topobject, "pcb", "pcbnet");
 	break;
      case 3:
-        gennet("flatspice", "fspc");
+        writenet(topobject, "flatspice", "fspc");
 	break;
      case 4:
-	gennet("idxpcb", "");	/* Auto-numbering:  no output generated */
+	/* Auto-numbering:  no output generated */
+	writenet(topobject, "idxpcb", "");
 	break;
    }
 }
@@ -132,7 +132,7 @@ objectptr NameToObject(char *objname, objinstptr *ret_inst, Boolean dopages)
    for (i = 0; i < xobjs.numlibs; i++) {
       for (spec = xobjs.userlibs[i].instlist; spec != NULL; spec = spec->next) {
          if (!strcmp(objname, spec->thisinst->thisobject->name)) {
-	    *ret_inst = spec->thisinst;
+	    if (ret_inst) *ret_inst = spec->thisinst;
 	    return spec->thisinst->thisobject;
 	 }
       }
@@ -142,7 +142,7 @@ objectptr NameToObject(char *objname, objinstptr *ret_inst, Boolean dopages)
       for (i = 0; i < xobjs.pages; i++) {
 	 if (xobjs.pagelist[i]->pageinst == NULL) continue;
 	 if (!strcmp(objname, xobjs.pagelist[i]->pageinst->thisobject->name)) {
-	    *ret_inst = xobjs.pagelist[i]->pageinst;
+	    if (ret_inst) *ret_inst = xobjs.pagelist[i]->pageinst;
 	    return xobjs.pagelist[i]->pageinst->thisobject;
 	 }
       }
@@ -343,7 +343,7 @@ Boolean setobjecttype(objectptr cschem)
 
    /* Apply only to schematic objects */
 
-   if (cschem->schemtype != SCHEMATIC) {
+   if ((cschem->schemtype != PRIMARY) && (cschem->schemtype != SECONDARY)) {
       if (cschem->schemtype == FUNDAMENTAL)
 	 cschem->schemtype = SYMBOL;
       if (cschem->symschem == NULL) {
@@ -467,7 +467,7 @@ void setsymschem()
    /* Set menu items appropriately for this object */
 
    if (topobject->symschem != NULL) {
-      if (topobject->schemtype == SCHEMATIC) {
+      if (topobject->schemtype == PRIMARY || topobject->schemtype == SECONDARY) {
          XtSetArg(aargs[0], XtNlabel, "Go To Symbol");
          XtSetArg(bargs[0], XtNlabel, "Disassociate Symbol");
       }
@@ -477,7 +477,7 @@ void setsymschem()
       }
    }
    else {
-      if (topobject->schemtype == SCHEMATIC) {
+      if (topobject->schemtype == PRIMARY || topobject->schemtype == SECONDARY) {
          XtSetArg(aargs[0], XtNlabel, "Make Matching Symbol");
          XtSetArg(bargs[0], XtNlabel, "Associate with Symbol");
       }
@@ -491,7 +491,7 @@ void setsymschem()
 
    /* Set colors on the symbol and schematic buttons */
 
-   if (topobject->schemtype == SCHEMATIC) {
+   if (topobject->schemtype == PRIMARY || topobject->schemtype == SECONDARY) {
       if (topobject->symschem == NULL) {
          XtSetArg(aargs[0], XtNbackground, OFFBUTTONCOLOR);
 	 XtSetArg(aargs[1], XtNforeground, OFFBUTTONCOLOR);
@@ -605,27 +605,35 @@ int findsubschems(int toppage, objectptr cschem, int level, short *pagelist)
 
 void collectsubschems(int toppage)
 {
-   int level = 0;
+   int loctop, level = 0;
    objectptr cschem;
    short *pagelist, pageno;
+   Pagedata *curpage;
 
-   if (xobjs.pagelist[toppage]->pageinst == NULL) return;
-
-   cschem = xobjs.pagelist[toppage]->pageinst->thisobject;
+   loctop = toppage;
+   curpage = xobjs.pagelist[loctop];
+   if (curpage->pageinst == NULL) return;
+   cschem = curpage->pageinst->thisobject;
+   if (cschem->schemtype == SECONDARY) {
+      cschem = cschem->symschem;
+      loctop = is_page(cschem);
+      if (loctop < 0) return;
+      curpage = xobjs.pagelist[loctop];
+   }
 
    pagelist = (short *)malloc(xobjs.pages * sizeof(short));
 
    for (pageno = 0; pageno < xobjs.pages; pageno++)
       pagelist[pageno] = 0;
 
-   findsubschems(toppage, cschem, 0, pagelist);
+   findsubschems(loctop, cschem, 0, pagelist);
 
    for (pageno = 0; pageno < xobjs.pages; pageno++) {
       if (pagelist[pageno] > 0) {
 	  if (xobjs.pagelist[pageno]->filename != NULL)
 	     free(xobjs.pagelist[pageno]->filename);
 	  xobjs.pagelist[pageno]->filename =
-	     strdup(xobjs.pagelist[toppage]->filename);
+	     strdup(xobjs.pagelist[loctop]->filename);
       }
    } 
    free((char *)pagelist);
@@ -669,7 +677,7 @@ void copypinlabel(labelptr pinlab)
 	   * (*newlabel)->scale;
       schemobj->parts++;
       incr_changes(schemobj);
-      schemobj->valid = False;
+      invalidate_netlist(schemobj);
       singlebbox(ENDPART);
    }
 }
@@ -689,7 +697,7 @@ int checkschem(objectptr thisobj, char *cname)
    objectptr *tlib;
    short i, j;
 
-   if (areastruct.schemon == False || thisobj->symschem != NULL) return 0;
+   if ((areastruct.schemon == False) || (thisobj->symschem != NULL)) return 0;
 
    for (i = 0; i < xobjs.numlibs; i++) {
       for (j = 0; j < xobjs.userlibs[i].number; j++) {
@@ -697,7 +705,7 @@ int checkschem(objectptr thisobj, char *cname)
 
          if (!strcmp(cname, (*tlib)->name)) {
 	    thisobj->symschem = (*tlib);
-	    thisobj->schemtype = SCHEMATIC;
+	    thisobj->schemtype = PRIMARY;
 	    (*tlib)->symschem = thisobj;
 	    (*tlib)->schemtype = SYMBOL;
 	    return 1;
@@ -726,7 +734,7 @@ int checksym(objectptr symobj, char *cname)
 	    symobj->symschem = checkpage;
 	    symobj->schemtype = SYMBOL;
 	    checkpage->symschem = symobj;
-	    checkpage->schemtype = SCHEMATIC;
+	    checkpage->schemtype = PRIMARY;
 	    return 1;
 	 }
       }
@@ -780,12 +788,13 @@ void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
    if (areastruct.schemon == False) return;
 
    if (eventmode == PRESS_MODE || eventmode == COPY2_MODE)
-      objectdelete(NORMAL);
+      standard_element_delete(NORMAL);
 
    /* Create symbol or schematic, if allowed by mode */
 
-   if ((topobject->symschem == NULL) && (mode != 0)) {
-      if (topobject->schemtype != SCHEMATIC) {
+   if ((topobject->symschem == NULL) && (mode != 0)
+		&& (topobject->schemtype != SECONDARY)) {
+      if (topobject->schemtype != PRIMARY) {
 	 int tpage;
 
 	 /* create a new page for the new schematic */
@@ -970,7 +979,7 @@ void dobeforeswap(xcWidget w, caddr_t clientdata, caddr_t calldata)
 
    /* Check for requirement to change the name before creating the symbol */
 
-   if ((topobject->symschem == NULL) && (topobject->schemtype == SCHEMATIC)
+   if ((topobject->symschem == NULL) && (topobject->schemtype == PRIMARY)
 		&& (strstr(topobject->name, "Page ") != NULL)) {
 
       /* Get a name for the new object */
@@ -1016,9 +1025,12 @@ void startschemassoc(xcWidget w, pointertype mode, caddr_t calldata)
    else if ((topobject->symschem != NULL) && (mode == 0)) {
       Wprintf("Refusing to undo current association.");
    }
+   else if (topobject->schemtype == SECONDARY) {
+      Wprintf("Cannot attach symbol to a secondary schematic page.");
+   }
    else {
       eventmode = ASSOC_MODE;
-      if (topobject->schemtype == SCHEMATIC) {
+      if (topobject->schemtype == PRIMARY) {
 	 /* Find a symbol to associate */
 	 startcatalog(w, LIBLIB, NULL);
 	 Wprintf("Click on library page, then symbol to associate.");
@@ -1067,5 +1079,4 @@ Boolean schemassoc(objectptr schemobj, objectptr symbolobj)
    return True;
 }
 
-#endif
 /*-------------------------------------------------------------------------*/
