@@ -117,7 +117,7 @@ void joinlabels()
 	 for (; endpart->nextpart != NULL; endpart = endpart->nextpart);
 	 free(source);
 	 removep(jl, 0);
-         reviseselect(jl);
+         reviseselect(areastruct.selectlist, areastruct.selects, jl);
       }
    }
 
@@ -952,10 +952,8 @@ void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
 
    if (fontcount == 0) return;
 
-#ifdef SCHEMA
    /* Don't draw temporary labels from schematic capture system */
    if (drawlabel->string->type != FONT_NAME) return;
-#endif
 
    if (passcolor == DOSUBSTRING)
       defaultcolor = curcolor = drawlabel->color;
@@ -990,7 +988,6 @@ void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
        (tmpjust & TOP ? -tmpext.ascent : -(tmpext.ascent + tmpext.base) >> 1)
 		: -tmpext.base);
 
-#ifdef SCHEMA
    /* Pinlabels have an additional offset spacing to pad */
    /* them from the circuit point to which they attach.  */
 
@@ -1000,7 +997,6 @@ void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
       else
 	 pinadjust(tmpjust, &(newpoint.x), &(newpoint.y), 1);
    }
-#endif
 
    oldx = newpoint.x;
    baseline = newpoint.y;
@@ -1191,9 +1187,7 @@ enddraw:
 
    UPopCTM();
 
-#ifdef SCHEMA
    if (drawlabel->pin && areastruct.schemon) UDrawXDown(drawlabel);
-#endif
 
    if ((defaultcolor != DOFORALL) && (passcolor != curcolor)) {
       XTopSetForeground(passcolor);
@@ -1223,10 +1217,9 @@ TextExtents ULength(stringpart *string, objinstptr localinst, float newscale,
    retext.width = retext.ascent = retext.descent = retext.base = 0;
 
    if (fontcount == 0) return retext;
-#ifdef SCHEMA
+
    /* Don't draw temporary labels from schematic capture system */
    else if (string->type != FONT_NAME) return retext;
-#endif
 
    if (newscale > 0.0) natscale = UTopScale() * newscale;
    else natscale = 1.0;
