@@ -38,7 +38,8 @@ extern int xctcl_promptsavelib();
 extern int xctcl_promptsavepage();
 extern int xctcl_eventmode();
 extern int xctcl_delete();
-extern int xctcl_undelete();
+extern int xctcl_undo();
+extern int xctcl_redo();
 extern int xctcl_copy();
 extern int xctcl_move();
 extern int xctcl_edit();
@@ -63,10 +64,8 @@ extern int xctcl_arc();
 extern int xctcl_object();
 extern int xctcl_here();
 extern int xctcl_start();
-#ifdef SCHEMA
 extern int xctcl_netlist();
 extern int xctcl_symschem();
-#endif
 extern int xctcl_tag();
 
 typedef struct {
@@ -93,7 +92,8 @@ static cmdstruct xc_commands[] =
    {"promptsavepage", (void *)xctcl_promptsavepage},
    {"eventmode", (void *)xctcl_eventmode},
    {"delete", (void *)xctcl_delete},
-   {"undelete", (void *)xctcl_undelete},
+   {"undo", (void *)xctcl_undo},
+   {"redo", (void *)xctcl_redo},
    {"select", (void *)xctcl_select},
    {"deselect", (void *)xctcl_deselect},
    {"copy", (void *)xctcl_copy},
@@ -117,11 +117,9 @@ static cmdstruct xc_commands[] =
    {"arc", (void *)xctcl_arc},
    {"object", (void *)xctcl_object},
    {"here", (void *)xctcl_here},
-#ifdef SCHEMA
    {"netlist", (void *)xctcl_netlist},
    {"symbol", (void *)xctcl_symschem},
    {"schematic", (void *)xctcl_symschem},
-#endif
    {"", NULL}  /* sentinel */
 };
 
@@ -139,6 +137,7 @@ Xcircuit_Init(Tcl_Interp *interp)
    Tk_Window tktop;
    char *tmp_s;
    char *cadhome;
+   char version_string[20];
 
    /* Interpreter sanity checks */
    if (interp == NULL) return TCL_ERROR;
@@ -198,10 +197,20 @@ Xcircuit_Init(Tcl_Interp *interp)
    if (cadhome == NULL) cadhome = CAD_DIR;
    Tcl_SetVar(interp, "CAD_HOME", cadhome, TCL_GLOBAL_ONLY);
 
+   /* Set $XCIRCUIT_VERSION as a Tcl variable */
+
+   sprintf(version_string, "%g", PROG_VERSION);
+   Tcl_SetVar(interp, "XCIRCUIT_VERSION", version_string, TCL_GLOBAL_ONLY);
+
+#ifdef ASG
+   /* Set a variable in Tcl indicating that the ASG module is compiled in */
+   Tcl_SetVar(interp, "XCIRCUIT_ASG", "1", TCL_GLOBAL_ONLY);
+#endif
+
    /* Export the namespace commands */
 
    Tcl_Eval(interp, "namespace eval xcircuit namespace export *");
-   Tcl_PkgProvide(interp, "Xcircuit", "3.0");
+   Tcl_PkgProvide(interp, "Xcircuit", version_string);
 
    /* Initialize the console interpreter, if there is one. */
 
